# -*- coding: UTF-8 -*-
"""
Tests
=====
@ Dash Picture Annotation

Author
------
Yuchen Jin (cainmagi)
cainmagi@gmail.com

Description
-----------
Testing codes powered by `pytest`. This namespace module is used for providing the
unit tests for the package.
"""
