.dropdown {
  width: 120%;
}

.dropdown.row {
  display: flex !important;
}

.dropdown.row > * {
  margin-top: 0;
}

.dropdown.row > *:not(:last-child) {
  padding-right: 0.75rem;
}

.dropdown.row > .col-main {
  flex-grow: 1 !important;
}

.dropdown.row > .col-btn {
  padding: 0;
}

.btn-delete {
  -webkit-box-align: center;
  -webkit-box-pack: justify;
  float: right;
  align-items: center;
  cursor: default;
  justify-content: space-between;
  max-width: min-content;
  min-height: 38px;
  padding-left: 0;
  padding-right: 0;
  position: relative;
  transition: 100ms;
  background-color: rgb(255, 255, 255);
  border-color: rgb(204, 204, 204);
  border-radius: 4px;
  border-style: solid;
  border-width: 1px;
  box-sizing: border-box;
  outline: 0px !important;
}

.icon-delete {
  display: flex;
  transition: color 150ms;
  color: rgb(204, 204, 204);
  padding: 8px;
  box-sizing: border-box;
}

.icon-delete:hover {
  color: rgb(153, 153, 153);
}

.icon-delete:focus {
  color: rgb(51, 51, 51);
}
