# -*- coding: UTF-8 -*-
"""
Setup
=====

Author
------
Yuchen Jin (cainmagi)
cainmagi@gmail.com

Description
-----------
Dummy `setup.py` for ensuring the compatibility. The configurations are made in
`pyproject.toml`.

Run the following commands to build the package/wheel.
``` shell
python -m build
```
"""

from setuptools import setup

setup()
